/*
Module : PyDebugCallStackWnd.cpp
Purpose: Defines the implementation for the classes which encapsulate the Call Stack window functionality of PyDebug++
Created: PJN / 20-08-2015
History: PJN / 01-08-2022 1. Updated all the code in to use C++ uniform initialization for all variable
                          declarations.

Copyright (c) 2015 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "PyDebugCallStackWnd.h"
#include "PyDebugMainFrm.h"
#include "resource.h"


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG

constexpr int CALLSTACK_LIST_CTRL_ID = 1;


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CPyDebugCallStackListCtrl, CListCtrl) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_CREATE()
#pragma warning(suppress: 26454)
  ON_NOTIFY_REFLECT(NM_DBLCLK, &CPyDebugCallStackListCtrl::OnNMDblclk)
END_MESSAGE_MAP()

#pragma warning(suppress: 26429 26461)
void CPyDebugCallStackListCtrl::OnNMDblclk(_In_ NMHDR* pNMHDR, _Inout_ LRESULT* pResult)
{
#pragma warning(suppress: 26490)
  const auto* pItem{reinterpret_cast<NMITEMACTIVATE*>(pNMHDR)};
  ASSERT(pItem != nullptr);

  if (pItem->iItem >= 0)
  {
    //Ask the frame window to navigate to the specified breakpoint
    ASSERT(m_pMainFrame != nullptr);
    CSingleLock sl{&m_pMainFrame->m_csStackFrames, TRUE};
#pragma warning(suppress: 26446)
    const CPyDebugStackFrame& frame{m_pMainFrame->m_StackFrames[pItem->iItem]};
    m_pMainFrame->NavigateToDocumentLine(frame.m_sModule, frame.m_LineNumber - 1, TRUE);
  }

  *pResult = 0;
}


#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CPyDebugCallStackWnd, CDockablePane) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_CREATE()
  ON_WM_SIZE()
  ON_WM_SETFOCUS()
END_MESSAGE_MAP()

#pragma warning(suppress: 26434)
int CPyDebugCallStackWnd::OnCreate(_In_ LPCREATESTRUCT lpCreateStruct)
{
  //Let the base class do its thing
  if (__super::OnCreate(lpCreateStruct) == -1)
    return -1;

  //Create the list control
  CRect rectDummy;
  rectDummy.SetRectEmpty();
  if (!m_wndCtrl.Create(WS_CHILD | WS_VISIBLE | LVS_REPORT | LVS_SHOWSELALWAYS, rectDummy, this, CALLSTACK_LIST_CTRL_ID))
  {
    TRACE(_T("Failed to create Call stack window\n"));
    return -1;
  }
  m_wndCtrl.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP | LVS_EX_GRIDLINES);
  m_wndCtrl.EnableWindow(FALSE);

  //Create the image list
  if (!m_ImageList.Create(IDB_PYDEBUG_CALLSTACK, 16, 1, RGB(255, 0, 255)))
  {
    TRACE(_T("Failed to create Call stack window image list\n"));
    return -1;
  }
  m_wndCtrl.SetImageList(&m_ImageList, LVSIL_SMALL);

  //Add the columns to the list control
  CRect rClient;
  GetClientRect(&rClient);
  CString sColumn;
  if (!sColumn.LoadString(IDS_PYDEBUG_MODULE))
  {
    TRACE(_T("Failed to load Call stack window column caption\n"));
    return -1;
  }
  m_wndCtrl.InsertColumn(0, sColumn, LVCFMT_LEFT, 400);
  if (!sColumn.LoadString(IDS_PYDEBUG_FUNCTION))
  {
    TRACE(_T("Failed to load Call stack window column caption\n"));
    return -1;
  }
  m_wndCtrl.InsertColumn(1, sColumn, LVCFMT_LEFT, 400);
  if (!sColumn.LoadString(IDS_PYDEBUG_LINE_NUMBER))
  {
    TRACE(_T("Failed to load Call stack window column caption\n"));
    return -1;
  }
  m_wndCtrl.InsertColumn(2, sColumn, LVCFMT_LEFT, 150);

  return 0;
}

#pragma warning(suppress: 26434)
void CPyDebugCallStackWnd::OnSize(_In_ UINT nType, _In_ int cx, _In_ int cy)
{
  //Let the base class do its thing
  __super::OnSize(nType, cx, cy);

  //Make the control window covers the whole client area
  m_wndCtrl.SetWindowPos(nullptr, -1, -1, cx, cy, SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
}

#pragma warning(suppress: 26434)
void CPyDebugCallStackWnd::OnSetFocus(_In_opt_ CWnd* pOldWnd)
{
  //Let the base class do its thing
  __super::OnSetFocus(pOldWnd);

  //Give the focus to the control
  m_wndCtrl.SetFocus();
}

void CPyDebugCallStackWnd::SePyDebugMainFrame(_In_opt_ CPyDebugFrameWnd* pMainFrame) noexcept
{
  m_pMainFrame = pMainFrame;
  m_wndCtrl.m_pMainFrame = pMainFrame;
}
