/*
Module : PyDebugDoc.h
Purpose: Defines the interface for the classes which encapsulate the MFC Document functionality of PyDebug++
Created: PJN / 20-08-2015

Copyright (c) 2015 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once


#ifndef __PYDEBUGDOC_H__
#define __PYDEBUGDOC_H__

#ifndef CPYDEBUG_EXT_CLASS
#define CPYDEBUG_EXT_CLASS
#endif //#ifndef CPYDEBUG_EXT_CLASS


//////////////////// Includes /////////////////////////////////////////////////

#include "ScintillaDocView.h"
#include "PyDebugMainFrm.h"


//////////////////// Classes //////////////////////////////////////////////////

class CPYDEBUG_EXT_CLASS CPyDebugDoc : public Scintilla::CScintillaDoc
{
protected: //create from serialization only
  CPyDebugDoc() noexcept {}; //NOLINT(modernize-use-equals-default)

public:
//Constructors / Destructors
  CPyDebugDoc(const CPyDebugDoc&) = delete;
  CPyDebugDoc(CPyDebugDoc&&) = delete;
  ~CPyDebugDoc() = default; //NOLINT(modernize-use-override)

//Methods
  CPyDebugDoc& operator=(const CPyDebugDoc&) = delete;
  CPyDebugDoc& operator=(CPyDebugDoc&&) = delete;
  void DisableAllBreakpoints();
  void DeleteAllBreakpoints();
  void NavigateToLine(_In_ Scintilla::Line lineNumber);
  void UpdateBreakpoint(_In_ const CPyDebugBreakpoint& breakpoint);
  void DeleteBreakpoint(_In_ const CPyDebugBreakpoint& breakpoint);
  void UpdateBreakpointsFromUI();
  void UpdateBreakpointsToUI(_In_z_ LPCTSTR lpszPathName);

protected:
//Methods
  void DeleteContents() override;
  BOOL OnOpenDocument(_In_z_ LPCTSTR lpszPathName) override;
  CPyDebugFrameWnd* GetMainFrameWnd();

  DECLARE_MESSAGE_MAP()
  DECLARE_DYNCREATE(CPyDebugDoc)

  friend class CPyDebugView;
};


#endif //#ifndef __PYDEBUGDOC_H__
