/*
Module : PyDebugDocTemplate.cpp
Purpose: Defines the implementation for the class which implements the MFC Doc template functionality of PyDebug++
Created: PJN / 20-08-2015
History: PJN / 01-09-2015 1. Initial public release.
         PJN / 19-12-2015 1. Class initially created
         PJN / 03-04-2017 1. Fixed a bug where the documents would not appear in the application MRU
         PJN / 01-08-2022 1. Updated all the code in to use C++ uniform initialization for all variable
                          declarations.

Copyright (c) 2015 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "PyDebugDocTemplate.h"
#include "PyDebugMainFrm.h"


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


//////////////////// Implementation ///////////////////////////////////////////

CPyDebugMultiDocTemplate::CPyDebugMultiDocTemplate(_In_ UINT nIDResource, _In_ CRuntimeClass* pDocClass, _In_ CRuntimeClass* pFrameClass, _In_ CRuntimeClass* pViewClass) : CMultiDocTemplate{nIDResource, pDocClass, pFrameClass, pViewClass},
                                                                                                                                                                            m_pFrameWnd{nullptr}
{
}

CFrameWnd* CPyDebugMultiDocTemplate::CreateNewFrame(CDocument* pDoc, CFrameWnd* pOther)
{
  //Validate our parameters
  ASSERT(m_pFrameWnd != nullptr); //Did you forget to call SetPyDebugFrameWnd?

  if (pDoc != nullptr)
    ASSERT_VALID(pDoc);

  //create a frame wired to the specified document
  ASSERT(m_nIDResource != 0); //must have a resource ID to load from
  CCreateContext context;
  context.m_pCurrentFrame = pOther;
  context.m_pCurrentDoc = pDoc;
  context.m_pNewViewClass = m_pViewClass;
  context.m_pNewDocTemplate = this;
  context.m_pLastView = nullptr;

  if (m_pFrameClass == nullptr)
  {
    ASSERT(FALSE);
    return nullptr;
  }
#pragma warning(suppress: 26466)
  auto pFrame{static_cast<CFrameWnd*>(m_pFrameClass->CreateObject())};
  if (pFrame == nullptr)
    return nullptr;
  ASSERT_KINDOF(CFrameWnd, pFrame);

  //create new from resource
  if (!pFrame->LoadFrame(m_nIDResource,
                         WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE,  //default frame styles
                         m_pFrameWnd, &context))                //This is the critical change compared to CMultiDocTemplate::CreateNewFrame where we pass "m_pFrameWnd" instead of null
  {
    //frame will be deleted in PostNcDestroy cleanup
    return nullptr;
  }

  //it worked!
  return pFrame;
}

void CPyDebugMultiDocTemplate::SetPyDebugFrameWnd(_In_ CPyDebugFrameWnd* pFrameWnd) noexcept
{
  m_pFrameWnd = pFrameWnd;
}

CDocument* CPyDebugMultiDocTemplate::OpenDocumentFile(LPCTSTR lpszPathName, BOOL bMakeVisible)
{
  return __super::OpenDocumentFile(lpszPathName, TRUE, bMakeVisible);
}
