/*
Module : PyDebugModulesWnd.cpp
Purpose: Defines the implementation for the classes which encapsulate the Modules window functionality of PyDebug++
Created: PJN / 20-08-2015
History: PJN / 01-08-2022 1. Updated all the code in to use C++ uniform initialization for all variable
                          declarations.

Copyright (c) 2015 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "PyDebugModulesWnd.h"
#include "resource.h"
#include "PyDebugMainFrm.h"


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CPyDebugModulesWnd, CDockablePane) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_CREATE()
  ON_WM_SIZE()
  ON_WM_SETFOCUS()
END_MESSAGE_MAP()

#pragma warning(suppress: 26434)
int CPyDebugModulesWnd::OnCreate(_In_ LPCREATESTRUCT lpCreateStruct)
{
  //Let the base class do its thing
  if (__super::OnCreate(lpCreateStruct) == -1)
    return -1;

  //Create the list control
  CRect rectDummy;
  rectDummy.SetRectEmpty();
  if (!m_wndCtrl.Create(WS_CHILD | WS_VISIBLE | LVS_REPORT | LVS_SHOWSELALWAYS, rectDummy, this, 2))
  {
    TRACE(_T("Failed to create Modules window\n"));
    return -1;
  }
  m_wndCtrl.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP | LVS_EX_GRIDLINES);
  m_wndCtrl.EnableWindow(FALSE);

  //Create the image list
  BOOL bImageListCreated{FALSE};
  ASSERT(m_pMainFrame != nullptr); //Did you forget to call SetMainFrame
  const bool bHiColorIcons{m_pMainFrame->GetHiColorIcons()};
  if (bHiColorIcons)
  {
    bImageListCreated = m_ImageList.Create(16, 16, ILC_MASK | ILC_COLOR24, 0, 0);
    if (bImageListCreated)
    {
      CBitmap bmp;
      bImageListCreated = bmp.LoadBitmap(IDB_PYDEBUG_MODULES_HC);
      if (bImageListCreated)
        m_ImageList.Add(&bmp, RGB(255, 0, 255));
    }
  }
  else
    bImageListCreated = m_ImageList.Create(IDB_PYDEBUG_MODULES, 16, 1, RGB(255, 0, 255));
  if (!bImageListCreated)
  {
    TRACE(_T("Failed to create Modules window image list\n"));
    return -1;
  }
  m_wndCtrl.SetImageList(&m_ImageList, LVSIL_SMALL);

  //Add the columns to the list control
  CRect rClient;
  GetClientRect(&rClient);
  CString sColumn;
  if (!sColumn.LoadString(IDS_PYDEBUG_NAME))
  {
    TRACE(_T("Failed to load Modules window column caption\n"));
    return -1;
  }
  m_wndCtrl.InsertColumn(0, sColumn, LVCFMT_LEFT, 200);
  if (!sColumn.LoadString(IDS_PYDEBUG_DETAILS))
  {
    TRACE(_T("Failed to load Modules window column caption\n"));
    return -1;
  }
  m_wndCtrl.InsertColumn(1, sColumn, LVCFMT_LEFT, 800);

  return 0;
}

#pragma warning(suppress: 26434)
void CPyDebugModulesWnd::OnSize(_In_ UINT nType, _In_ int cx, _In_ int cy)
{
  //Let the base class do its thing
  __super::OnSize(nType, cx, cy);

  //Make the control window covers the whole client area
  m_wndCtrl.SetWindowPos(nullptr, -1, -1, cx, cy, SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
}

#pragma warning(suppress: 26434)
void CPyDebugModulesWnd::OnSetFocus(_In_opt_ CWnd* pOldWnd)
{
  //Let the base class do its thing
  __super::OnSetFocus(pOldWnd);

  //Give the focus to the control
  m_wndCtrl.SetFocus();
}

void CPyDebugModulesWnd::SetPyDebugMainFrame(_In_opt_ CPyDebugFrameWnd* pMainFrame) noexcept
{
  m_pMainFrame = pMainFrame;
}
