/*
Module : PyDebugOutputWnd.cpp
Purpose: Defines the implementation for the classes which encapsulate the Output window functionality of PyDebug++
Created: PJN / 20-08-2015
History: PJN / 01-08-2022 1. Updated all the code in to use C++ uniform initialization for all variable
                          declarations.

Copyright (c) 2015 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "PyDebugOutputWnd.h"
#include "resource.h"
#include "PyDebugMainFrm.h"


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


//////////////////// Implementation ///////////////////////////////////////////

BOOL CPyDebugOutputWndDoc::IsModified() noexcept
{
  return FALSE;
}


#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CPyDebugOutputWnd, CDockablePane) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_CREATE()
  ON_WM_SIZE()
  ON_WM_SETFOCUS()
  ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()

CPyDebugOutputWnd::CPyDebugOutputWnd() noexcept : m_pWndView{nullptr}
{
  m_Document.m_bAutoDelete = FALSE;
}

#pragma warning(suppress: 26434)
int CPyDebugOutputWnd::OnCreate(_In_ LPCREATESTRUCT lpCreateStruct)
{
  //Let the base class do its thing
  if (__super::OnCreate(lpCreateStruct) == -1)
    return -1;

  //Create the scintilla view
  ASSERT(m_pWndView == nullptr);
  CCreateContext context;
  context.m_pCurrentFrame = nullptr;
  context.m_pCurrentDoc = &m_Document;
  context.m_pNewViewClass = RUNTIME_CLASS(Scintilla::CScintillaView);
  context.m_pNewDocTemplate = nullptr;
  context.m_pLastView = nullptr;
#pragma warning(suppress: 26466)
  m_pWndView = static_cast<Scintilla::CScintillaView*>(context.m_pNewViewClass->CreateObject());
  if (m_pWndView == nullptr)
    return -1;
  if (!m_pWndView->Create(nullptr, nullptr, WS_CHILD | WS_VISIBLE, CRect{0,0,0,0}, this, 2, &context))
    return -1;
  auto& rCtrl{m_pWndView->GetCtrl()};
  rCtrl.SetReadOnly(TRUE);
  rCtrl.SetMarginWidthN(1, 0);
  rCtrl.UsePopUp(Scintilla::PopUp::Never);

  return 0;
}

#pragma warning(suppress: 26434)
void CPyDebugOutputWnd::OnSize(_In_ UINT nType, _In_ int cx, _In_ int cy)
{
  //Validate our parameters
  ASSERT(m_pWndView != nullptr);

  //Let the base class do its thing
  __super::OnSize(nType, cx, cy);

  //Make the control window covers the whole client area
  m_pWndView->SetWindowPos(nullptr, -1, -1, cx, cy, SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
}

void CPyDebugOutputWnd::AddOutputText(_In_z_ LPCTSTR pszText)
{
  //Validate our parameters
  ASSERT(m_pWndView != nullptr);

  auto& rCtrl{m_pWndView->GetCtrl()};
  rCtrl.SetReadOnly(FALSE);
#pragma warning(suppress: 4267 26472)
  rCtrl.AppendText(static_cast<Sci_Position>(_tcslen(pszText)), pszText);
  rCtrl.SetReadOnly(TRUE);
}

#pragma warning(suppress: 26434)
void CPyDebugOutputWnd::OnSetFocus(_In_opt_ CWnd* pOldWnd)
{
  //Validate our parameters
  ASSERT(m_pWndView != nullptr);

  //Let the base class do its thing
  __super::OnSetFocus(pOldWnd);

  //Give the focus to the view
  m_pWndView->SetFocus();
}

#pragma warning(suppress: 26434)
void CPyDebugOutputWnd::OnContextMenu(_In_opt_ CWnd* /*pWnd*/, _In_ CPoint point)
{
  //Validate our parameters
  ASSERT(m_pWndView != nullptr);

  //Load up the context menu we will be using
  CMenu menu;
  if (!menu.LoadMenu(IDR_PYDEBUG_POPUP_OUTPUT))
    return;
#pragma warning(suppress: 26429)
  CMenu* pPopup{menu.GetSubMenu(0)};
  ASSERT(pPopup != nullptr);

  //Display the menu
  auto& rCtrl{m_pWndView->GetCtrl()};
  const Sci_Position nStartChar{rCtrl.GetSelectionStart()};
  const Sci_Position nEndChar{rCtrl.GetSelectionEnd()};
  pPopup->EnableMenuItem(ID_EDIT_COPY, nStartChar != nEndChar ? MF_BYCOMMAND | MF_ENABLED : MF_BYCOMMAND | MF_DISABLED);
#pragma warning(suppress: 26472)
  const auto nCmd{static_cast<UINT>(pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON | TPM_RETURNCMD, point.x, point.y, m_pWndView))};
  switch (nCmd)
  {
    case ID_EDIT_COPY:
    {
      rCtrl.Copy();
      break;
    }
    case ID_EDIT_CLEAR:
    {
      rCtrl.SetReadOnly(FALSE);
      rCtrl.SelectAll();
      rCtrl.Clear();
      rCtrl.SetReadOnly(TRUE);
      break;
    }
    case ID_EDIT_SELECT_ALL:
    {
      rCtrl.SelectAll();
      break;
    }
    default:
    {
      break;
    }
  }
}
