/*
Module : PyDebugOutputWnd.h
Purpose: Defines the interface for the classes which encapsulate the Output window functionality of PyDebug++
Created: PJN / 20-08-2015

Copyright (c) 2015 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __PYDEBUGOUTPUTWND_H__
#define __PYDEBUGOUTPUTWND_H__

#ifndef CPYDEBUG_EXT_CLASS
#define CPYDEBUG_EXT_CLASS
#endif //#ifndef CPYDEBUG_EXT_CLASS


//////////////////// Includes /////////////////////////////////////////////////

#include "ScintillaDocView.h"


//////////////////// Classes //////////////////////////////////////////////////

class CPYDEBUG_EXT_CLASS CPyDebugOutputWndDoc : public Scintilla::CScintillaDoc
{
public:
//Methods
  BOOL IsModified() noexcept override;
};


class CPYDEBUG_EXT_CLASS CPyDebugOutputWnd : public CDockablePane
{
public:
//Constructors / Destructors
  CPyDebugOutputWnd() noexcept;
  CPyDebugOutputWnd(const CPyDebugOutputWnd&) = delete;
  CPyDebugOutputWnd(CPyDebugOutputWnd&&) = delete;
  ~CPyDebugOutputWnd() = default; //NOLINT(modernize-use-override)

//Methods
  CPyDebugOutputWnd& operator=(const CPyDebugOutputWnd&) = delete;
  CPyDebugOutputWnd& operator=(CPyDebugOutputWnd&&) = delete;
  void AddOutputText(_In_z_ LPCTSTR pszText);

protected:
//Member variables
  Scintilla::CScintillaView* m_pWndView;
  CPyDebugOutputWndDoc m_Document;

//Message handlers
  afx_msg int OnCreate(_In_ LPCREATESTRUCT lpCreateStruct);
  afx_msg void OnSize(_In_ UINT nType, _In_ int cx, _In_ int cy);
  afx_msg void OnSetFocus(_In_opt_ CWnd* pOldWnd);
  afx_msg void OnContextMenu(_In_opt_ CWnd* pWnd, _In_ CPoint point);

  DECLARE_MESSAGE_MAP()

  friend class CPyDebugFrameWnd;
};


#endif //#ifndef __PYDEBUGOUTPUTWND_H__
