#include "stdafx.h"
#include "afxwinappex.h"
#include "afxdialogex.h"
#include "PyDebugPP.h"
#include "PyDebugMainFrm.h"
#include "PyDebugDoc.h"
#include "PyDebugView.h"
#include "PyDebugChildFrm.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG

//The one and only CPyDebugPPApp object
#pragma warning(suppress: 26426)
CPyDebugPPApp theApp;


class CAboutDlg : public CDialogEx
{
public:
//Constructors / Destructors
  CAboutDlg();
  CAboutDlg(const CAboutDlg&) = delete;
  CAboutDlg(CAboutDlg&&) = delete;
  ~CAboutDlg() = default; //NOLINT(modernize-use-override)

//Methods
  CAboutDlg& operator=(const CAboutDlg&) = delete;
  CAboutDlg& operator=(CAboutDlg&&) = delete;

//Dialog Data
  enum { IDD = IDD_ABOUTBOX };

protected:
//Methods
  void DoDataExchange(CDataExchange* pDX) override;

  DECLARE_MESSAGE_MAP()
};


#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CPyDebugPPApp, CWinAppEx) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
  ON_COMMAND(ID_APP_ABOUT, &CPyDebugPPApp::OnAppAbout)
  ON_COMMAND(ID_FILE_NEW, &CWinAppEx::OnFileNew)
  ON_COMMAND(ID_FILE_OPEN, &CWinAppEx::OnFileOpen)
  ON_COMMAND(ID_FILE_PRINT_SETUP, &CWinAppEx::OnFilePrintSetup)
END_MESSAGE_MAP()

#pragma warning(suppress: 26455)
CPyDebugPPApp::CPyDebugPPApp() : m_pCreateLexer{nullptr},
                                 m_hScintilla{nullptr},
                                 m_hLexilla{nullptr},
                                 m_pPythonThreadState{nullptr},
                                 m_pyConfig{}
{
  //Support Restart Manager
  m_dwRestartManagerSupportFlags = AFX_RESTART_MANAGER_SUPPORT_ALL_ASPECTS;

  SetAppID(_T("PyDebugPP.AppID.NoVersion"));
}

HMODULE CPyDebugPPApp::LoadLibraryFromApplicationDirectory(_In_z_ LPCTSTR lpFileName)
{
  //Get the Application directory
  CString sFullPath;
  const DWORD dwGMFN{GetModuleFileName(nullptr, sFullPath.GetBuffer(_MAX_PATH), _MAX_PATH)};
  sFullPath.ReleaseBuffer();
  if (dwGMFN == 0)
#pragma warning(suppress: 26487)
    return nullptr;

  //Form the new path
  CString sDrive;
  CString sDir;
  _tsplitpath_s(sFullPath, sDrive.GetBuffer(_MAX_DRIVE), _MAX_DRIVE, sDir.GetBuffer(_MAX_DIR), _MAX_DIR, nullptr, 0, nullptr, 0);
  sDir.ReleaseBuffer();
  sDrive.ReleaseBuffer();
  CString sFname;
  CString sExt;
  _tsplitpath_s(lpFileName, nullptr, 0, nullptr, 0, sFname.GetBuffer(_MAX_FNAME), _MAX_FNAME, sExt.GetBuffer(_MAX_EXT), _MAX_EXT);
  sExt.ReleaseBuffer();
  sFname.ReleaseBuffer();
  _tmakepath_s(sFullPath.GetBuffer(_MAX_PATH), _MAX_PATH, sDrive, sDir, sFname, sExt);
  sFullPath.ReleaseBuffer();

  //Delegate to LoadLibrary
#pragma warning(suppress: 26487)
  return LoadLibrary(sFullPath);
}

BOOL CPyDebugPPApp::InitInstance()
{
  //Initialize the common controls we need
  INITCOMMONCONTROLSEX InitCtrls;
  InitCtrls.dwSize = sizeof(InitCtrls);
  InitCtrls.dwICC = ICC_WIN95_CLASSES;
  InitCommonControlsEx(&InitCtrls);

  //Load the Scintilla \ Lexilla dlls
  CString sLexillaDLL;
  sLexillaDLL.Format(_T("%s%s"), _T(LEXILLA_LIB), _T(LEXILLA_EXTENSION));
  m_hLexilla = LoadLibraryFromApplicationDirectory(sLexillaDLL);
  if (m_hLexilla == nullptr)
  {
    CString sMsg;
    sMsg.Format(_T("%s is not installed, Please built the Scintilla '%s' and copy it into this application's directory"), sLexillaDLL.GetString(), sLexillaDLL.GetString());
    AfxMessageBox(sMsg);
    return FALSE;
  }
  m_hScintilla = LoadLibraryFromApplicationDirectory(_T("Scintilla.dll"));
  if (m_hScintilla == nullptr)
  {
    AfxMessageBox(_T("Scintilla DLL is not installed, Please built the Scintilla 'Scintilla.dll' and copy it into this application's directory"));
    return FALSE;
  }
  //Get the CreateLexer function pointer from Lexilla
#pragma warning(suppress: 26490)
  m_pCreateLexer = reinterpret_cast<Lexilla::CreateLexerFn>(GetProcAddress(m_hLexilla, LEXILLA_CREATELEXER));
  if (m_pCreateLexer == nullptr)
  {
    AfxMessageBox(_T("Could not find the Lexilla CreateLexer function"));
    return FALSE;
  }

  //Setup Python
#pragma warning(suppress: 26486)
  PyConfig_InitIsolatedConfig(&m_pyConfig);
  const PyStatus pyStatus{Py_InitializeFromConfig(&m_pyConfig)};
  if (PyStatus_Exception(pyStatus))
  {
    CString sMsg;
#pragma warning(suppress: 26485 26486)
    sMsg.Format(_T("Failed to initialize Python from config. Python Exit Code:%d, Message:%s, Function:%s"), pyStatus.exitcode, (pyStatus.err_msg != nullptr) ? CString(pyStatus.err_msg).GetString() : _T("<NULL"), (pyStatus.func != nullptr) ? CString(pyStatus.func).GetString() : _T("<NULL>"));
    AfxMessageBox(sMsg);
    return FALSE;
  }
  m_pPythonThreadState = PyEval_SaveThread();

  //Let the base class do its thing
  if (!__super::InitInstance())
    return FALSE;

  EnableTaskbarInteraction(FALSE);
  SetRegistryKey(_T("Local AppWizard-Generated Applications"));
  LoadStdProfileSettings(4);  // Load standard INI file options (including MRU)
  InitContextMenuManager();
  InitShellManager();
  InitKeyboardManager();
  InitTooltipManager();
  CMFCToolTipInfo ttParams;
  ttParams.m_bVislManagerTheme = TRUE;
  theApp.GetTooltipManager()->SetTooltipParams(AFX_TOOLTIP_TYPE_ALL, RUNTIME_CLASS(CMFCToolTipCtrl), &ttParams);

  //Register the application's document templates. Document templates serve as the connection between documents, frame windows and views
#pragma warning(suppress: 26400 26403 26409)
  auto pDocTemplate{new CPyDebugMultiDocTemplate(IDR_PYDEBUG_PYTHON_DOCUMENT, RUNTIME_CLASS(CPyDebugDoc), RUNTIME_CLASS(CPyDebugChildFrame), RUNTIME_CLASS(CPyDebugView))};
  if (pDocTemplate == nullptr)
    return FALSE;
  AddDocTemplate(pDocTemplate);

  //create main MDI Frame window
#pragma warning(suppress: 26400 26403 26409)
  auto pMainFrame{new CPyDebugFrameWnd};
  ASSERT(pMainFrame != nullptr);
  if (!pMainFrame->LoadFrame(IDR_PYDEBUG_MAINFRAME))
    return FALSE;
  m_pMainWnd = pMainFrame;
  pDocTemplate->SetPyDebugFrameWnd(pMainFrame);

  //Parse command line for standard shell commands, DDE, file open
  CCommandLineInfo cmdInfo;
  ParseCommandLine(cmdInfo);

  //Enable DDE Execute open
  EnableShellOpen();
  RegisterShellFileTypes(TRUE);

  //Dispatch commands specified on the command line.  Will return FALSE if
  //app was launched with /RegServer, /Register, /Unregserver or /Unregister.
  if (!ProcessShellCommand(cmdInfo))
    return FALSE;

  //The one and only window has been initialized, so show and update it
  pMainFrame->SetDocTemplate(pDocTemplate);
  m_pMainWnd->ShowWindow(SW_SHOW);
  m_pMainWnd->UpdateWindow();

  //call DragAcceptFiles only if there's a suffix In an SDI app, this should occur after ProcessShellCommand
  m_pMainWnd->DragAcceptFiles();
  return TRUE;
}

int CPyDebugPPApp::ExitInstance()
{
  //Cleanup Python
  PyConfig_Clear(&m_pyConfig);
  if (m_pPythonThreadState != nullptr)
    PyEval_RestoreThread(m_pPythonThreadState);
  Py_FinalizeEx();

  //Unload the Scintilla \ Lexilla dlls
  if (m_hScintilla != nullptr)
  {
    FreeLibrary(m_hScintilla);
    m_hScintilla = nullptr;
  }
  if (m_hLexilla != nullptr)
  {
    FreeLibrary(m_hLexilla);
    m_hLexilla = nullptr;
  }

  //Let the base class do its thing
  return __super::ExitInstance();
}

void CPyDebugPPApp::OnAppAbout()
{
  CAboutDlg aboutDlg;
  aboutDlg.DoModal();
}


#pragma warning(suppress: 26455)
CAboutDlg::CAboutDlg() : CDialogEx{CAboutDlg::IDD}
{
}

#pragma warning(suppress: 26433)
void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  //Let the base class do its thing
  __super::DoDataExchange(pDX);
}

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CAboutDlg, CDialogEx) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
END_MESSAGE_MAP()
