/*
Module : PyDebugPropertiesWnd.h
Purpose: Defines the interface for the classes which encapsulate the Properties window functionality of PyDebug++
Created: PJN / 20-08-2015

Copyright (c) 2015 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __PYDEBUGPROPERTIESWND_H__
#define __PYDEBUGPROPERTIESWND_H__

#ifndef CPYDEBUG_EXT_CLASS
#define CPYDEBUG_EXT_CLASS
#endif //#ifndef CPYDEBUG_EXT_CLASS


//////////////////// Classes //////////////////////////////////////////////////

class CPyDebugFrameWnd;

class CPYDEBUG_EXT_CLASS CPyDebugPropertiesToolBar : public CMFCToolBar
{
public:
//Methods
  void OnUpdateCmdUI(CFrameWnd* /*pTarget*/, BOOL bDisableIfNoHndler) override
  {
#pragma warning(suppress: 26466)
    __super::OnUpdateCmdUI(static_cast<CFrameWnd*>(GetOwner()), bDisableIfNoHndler);
  }

  _NODISCARD BOOL AllowShowOnList() const noexcept override
  {
    return FALSE;
  }
};


class CPYDEBUG_EXT_CLASS CPyDebugPropertiesWnd : public CDockablePane
{
public:
//Methods
  void AdjustLayout() override;
  void SetPyDebugMainFrame(_In_opt_ CPyDebugFrameWnd* pMainFrame) noexcept;
  bool GetAllocateConsole();
  void SetAllocateConsole(_In_ bool bAllocateConsole);

protected:
//Member variables
  CFont m_fntPropList;
  CPyDebugPropertiesToolBar m_wndToolBar;
  CMFCPropertyGridCtrl m_wndPropList;
  CPyDebugFrameWnd* m_pMainFrame{nullptr};
  CMFCPropertyGridProperty* m_pAllocateConsole{nullptr};

//Methods
  void InitPropList();
  void SetPropListFont();

//Message handlers
  afx_msg int OnCreate(_In_ LPCREATESTRUCT lpCreateStruct);
  afx_msg void OnSize(_In_ UINT nType, _In_ int cx, _In_ int cy);
  afx_msg void OnExpandAllProperties();
  afx_msg void OnSortProperties();
  afx_msg void OnUpdateSortProperties(_In_ CCmdUI* pCmdUI);
  afx_msg void OnSetFocus(_In_opt_ CWnd* pOldWnd);
  afx_msg void OnSettingChange(_In_ UINT uFlags, _In_opt_z_ LPCTSTR lpszSection);

  DECLARE_MESSAGE_MAP()
};


#endif //#ifndef __PYDEBUGPROPERTIESWND_H__
