/*
Module : PyDebugView.h
Purpose: Defines the interface for the classes which encapsulate the MFC View functionality of PyDebug++
Created: PJN / 20-08-2015

Copyright (c) 2015 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __PYDEBUGVIEW_H__
#define __PYDEBUGVIEW_H__

#ifndef CPYDEBUG_EXT_CLASS
#define CPYDEBUG_EXT_CLASS
#endif //#ifndef CPYDEBUG_EXT_CLASS


//////////////////// Includes /////////////////////////////////////////////////

#include "ScintillaDocView.h"


//////////////////// Classes //////////////////////////////////////////////////

class CPYDEBUG_EXT_CLASS CPyDebugView : public Scintilla::CScintillaView
{
protected: //create from serialization only
  CPyDebugView() noexcept : m_pPythonLexer{nullptr} //NOLINT(modernize-use-equals-default)
  {
  };

public:
//Constructors / Destructors
  CPyDebugView(const CPyDebugView&) = delete;
  CPyDebugView(CPyDebugView&&) = delete;
  ~CPyDebugView() = default; //NOLINT(modernize-use-override)

//Methods
  CPyDebugView& operator=(const CPyDebugView&) = delete;
  CPyDebugView& operator=(CPyDebugView&&) = delete;
  _NODISCARD CPyDebugDoc* GetDocument() const;

protected:
//Member variables
  Scintilla::ILexer5* m_pPythonLexer;

//Methods
  void SetAStyle(_In_ int style, _In_ COLORREF fore, _In_ COLORREF back = RGB(0xFF, 0xFF, 0xFF), _In_ int size = -1, _In_opt_z_ const char* face = nullptr);
  void OnInitialUpdate() override;
  void OnModified(_Inout_ Scintilla::NotificationData* pSCNotification) override;

//Message handlers
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);

  DECLARE_MESSAGE_MAP()
  DECLARE_DYNCREATE(CPyDebugView)
};


#endif //#ifndef __PYDEBUGVIEW_H__
