/*
Module : PythonGILAutoLock.h
Purpose: Defines the interface for a simple C++ RAII class for acquiring and releasing the Python GIL
Created: PJN / 04-11-2017

Copyright (c) 2017 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __PYTHONGILAUTOLOCK_H__
#define __PYTHONGILAUTOLOCK_H__

#ifndef CPYDEBUG_EXT_CLASS
#define CPYDEBUG_EXT_CLASS
#endif //#ifndef CPYDEBUG_EXT_CLASS


//////////////////// Classes //////////////////////////////////////////////////

//Simple RAII wrapper for acquiring and releasing the Python GIL using PyGilState_Ensure & PyGILState_Release
class CPYDEBUG_EXT_CLASS CPyGILAutoLock
{
public:
//Constructors / Destructors
  CPyGILAutoLock() noexcept
  {
#pragma warning(suppress: 26812)
    m_state = PyGILState_Ensure();
  }
  CPyGILAutoLock(const CPyGILAutoLock&) = delete;
  CPyGILAutoLock(CPyGILAutoLock&&) = delete;
  ~CPyGILAutoLock() noexcept
  {
    PyGILState_Release(m_state);
  }

//Methods
  CPyGILAutoLock& operator=(const CPyGILAutoLock&) = delete;
  CPyGILAutoLock& operator=(CPyGILAutoLock&&) = delete;

//Member variables
  PyGILState_STATE m_state;
};

#endif //#ifndef __PYTHONGILAUTOLOCK_H__
